#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <math.h>
#include <stdio.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"


/* The loop types: */
static gint loop_int(gdouble t);
static gint loop_float(gdouble t);
static gint loop_sqrt(gdouble t);

static gint (*loop)(gdouble) = loop_int;


static gint current_rounds=0;
static GTimer *current_time_timer=NULL;

/* Avg storages */
static gint roundbuf[10]={0};
static gfloat timebuf[10]={0.000001};


/*** Simple callbacks: ***/


void on_calibrate_current_clicked(GtkButton *button, gpointer user_data) {
	calibrate_current(GTK_WIDGET(button));
	return;
}

void on_calibrate_average_clicked(GtkButton *button, gpointer user_data) {
	calibrate_average(GTK_WIDGET(button));
	return;
}


void on_looptype_changed(GtkToggleButton *togglebutton, gpointer user_data) {
	gchar *str=user_data;
	gint n;

	if (!GTK_TOGGLE_BUTTON(togglebutton)->active)
		return;

	if (g_strcasecmp(str,"int")==0) {
		loop=loop_int;
	} else if (g_strcasecmp(str,"float")==0) {
		loop=loop_float;
	} else if (g_strcasecmp(str,"sqrt")==0) {
		loop=loop_sqrt;
	} else {
		g_warning("on_looptype_changed called with argument %s!",str);
		return;
	}

	for (n=0; n<10; n++) {
		roundbuf[n]=0;
		timebuf[n]=0.000001;
	}
	g_timer_reset(current_time_timer);

	gtk_timeout_add(1000,calibrate_current,(gpointer)togglebutton);
	return;
}



/*** The stuff that actually does something: ***/


gboolean test_speed(gpointer data) {
	GtkWidget *parent=GTK_WIDGET(data);
	GtkWidget *spin,*label,*button;
	/* Values to set */
	gfloat current;
	gfloat avg;
	/* Some within-function-constants */
	gfloat current_time;
	gint rel_speed;
	/* Calc variables */
	gint avg_rounds;
	gfloat avg_time;
	/* Other */
	gint n;
	gchar buf[11];

	if (current_time_timer==NULL)
		return TRUE;

beginning:

	/* Get comparative speed */
	spin=lookup_widget(parent,"roundspeed");
	rel_speed=gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spin)) *
		1000000;

	current_time=g_timer_elapsed(current_time_timer,NULL);
	if (current_time<0.4)
		return TRUE;

	/* Calc the current speed */
	current=(gfloat)current_rounds/(current_time*rel_speed) - 1;

	/* Check if max-calibrate is on */
	if (current>0) {
		button=lookup_widget(parent,"max_toggle");
		if (GTK_TOGGLE_BUTTON(button)->active) {
			calibrate_current(parent);
			goto beginning;
		}
	}


	/* Move the averages */
	for (n=1; n<10; n++) {
		roundbuf[n-1]=roundbuf[n];
		timebuf[n-1]=timebuf[n];
	}
	roundbuf[9]=current_rounds;
	timebuf[9]=current_time;

	/* Calc the avg speed */
	avg_rounds=avg_time=0;
	for (n=0; n<10; n++) {
		avg_rounds+=roundbuf[n];
		avg_time+=timebuf[n];
	}
	avg=(gfloat)avg_rounds/(avg_time*rel_speed) - 1;

	/* Reset the stuff */
	current_rounds=0;
	g_timer_reset(current_time_timer);


	/* Set current speed */
	label=lookup_widget(parent,"speedcurrent");
	snprintf(buf,10,"%+.1f%%",current*100);
	gtk_label_set_text(GTK_LABEL(label),buf);

	/* Set avg speed */
	label=lookup_widget(parent,"speedavg");
	snprintf(buf,10,"%+.1f%%",avg*100);
	gtk_label_set_text(GTK_LABEL(label),buf);

	return TRUE;
}

/*
 * Calibrates on the requested data that we have.
 */
gboolean calibrate_current(gpointer data) {
	GtkWidget *parent=GTK_WIDGET(data);
	GtkWidget *spin;
	gfloat speed;
	gint cur_rounds;
	gfloat cur_time;

	if (current_time_timer==NULL)   /* timeout, not yet started */
		return TRUE;   /* Comeback! */

	cur_rounds=current_rounds;
	cur_time=g_timer_elapsed(current_time_timer,NULL);
	if (cur_time<0.4) {   /* If too short time elapsed, use previous */
		cur_rounds=roundbuf[9];
		cur_time=timebuf[9];
	}

	/* Calc the speed */
	speed=(gfloat)cur_rounds/(cur_time);
	speed/=1000000;

	if (speed>1000000) {
		g_warning("speed out of range! (Wow, what a fast machine!)");
	}
	speed=CLAMP(speed,0,1000000);

	spin=lookup_widget(parent,"roundspeed");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin),speed);

	return FALSE;
}
gboolean calibrate_average(gpointer data) {
	GtkWidget *parent=GTK_WIDGET(data);
	GtkWidget *spin;
	gfloat speed;
	gint avg_rounds;
	gfloat avg_time;
	gint n;

	/* Calc the avg speed */
	avg_rounds=avg_time=0;
	for (n=0; n<10; n++) {
		avg_rounds+=roundbuf[n];
		avg_time+=timebuf[n];
	}
	speed=(gfloat)avg_rounds/(avg_time);
	speed/=1000000;

	if (speed>1000000) {
		g_warning("speed out of range!");
	}
	speed=CLAMP(speed,0,1000000);

	spin=lookup_widget(parent,"roundspeed");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin),speed);

	return FALSE;
}


/*
 * The timeout function. Called at every occasion.
 */
gboolean loop_timeout(gpointer data) {

	if (current_time_timer==NULL) {
		current_time_timer=g_timer_new();
		g_timer_start(current_time_timer);
		current_rounds=0;
	}
	current_rounds+=loop(0.05);
	return TRUE;
}

/*** Loopers: ***/


static gint loop_int(gdouble t) {
	gint i,j;
	GTimer *timer;
	gulong a,b;

	timer=g_timer_new();
	g_timer_start(timer);

	a=12345;
	b=123;
	for (i=0; ; i++) {
		for (j=0; j<1000; j++)
			a*=b;

		if (g_timer_elapsed(timer,NULL)>=t)
			break;
	}

	sqrt(a);  /* Fool the compiler... */

	g_timer_destroy(timer);
	return (i*1000);
}


static gint loop_float(gdouble t) {
	gint i,j;
	GTimer *timer;
	gfloat a,b;

	timer=g_timer_new();
	g_timer_start(timer);

	a=123.345;
	b=23434.5436;
	for (i=0; ; i++) {
		for (j=0; j<1000; j++)
			a*=b;

		if (g_timer_elapsed(timer,NULL)>=t)
			break;
	}

	sqrt(a);  /* Fool the compiler... */

	g_timer_destroy(timer);
	return (i*1000);
}


static gint loop_sqrt(gdouble t) {
	gint i,j;
	GTimer *timer;
	gfloat a;

	timer=g_timer_new();
	g_timer_start(timer);

	a=13.132132;
	for (i=0; ; i++) {
		for (j=0; j<500; j++)  /* 500 for this has 2 sqrt's */
			a=sqrt(a)*sqrt(a+1);

		if (g_timer_elapsed(timer,NULL)>=t)
			break;
	}

	g_timer_destroy(timer);
	return (i*1000);
}

